@@ -1,7 +1,7 @@
 ### R code from vignette source 'sequenza.Rnw'
 
 ###################################################
-### code chunk number 1: sequenza.Rnw:59-60
+### code chunk number 1: sequenza.Rnw:60-61
 ###################################################
   options(width = 60, strict.width = "wrap")
 
@@ -60,7 +60,7 @@ seqz.data <- read.seqz(data.file)
 
 
 ###################################################
-### code chunk number 10: sequenza.Rnw:204-205
+### code chunk number 10: sequenza.Rnw:202-203
 ###################################################
 str(seqz.data, vec.len = 2)
 
@@ -161,7 +161,7 @@ par(mfrow = c(2,2))
 cp.plot(CP.example)
 cp.plot.contours(CP.example, add = TRUE)
 plot(cint$values.cellularity, ylab = "Cellularity",
-     xlab = "likelihood", type = "n")
+     xlab = "posterior probability", type = "n")
 select <- cint$confint.cellularity[1] <= cint$values.cellularity[,2] &
           cint$values.cellularity[,2] <= cint$confint.cellularity[2]
 polygon(y = c(cint$confint.cellularity[1], cint$values.cellularity[select, 2], cint$confint.cellularity[2]), 
@@ -170,7 +170,7 @@ lines(cint$values.cellularity)
 abline(h = cint$max.cellularity, lty = 2, lwd = 0.5)  
 
 plot(cint$values.ploidy, xlab = "Ploidy",
-     ylab = "likelihood", type = "n")
+     ylab = "posterior probability", type = "n")
 select <- cint$confint.ploidy[1] <= cint$values.ploidy[,1] &
           cint$values.ploidy[,1] <= cint$confint.ploidy[2]
 polygon(x = c(cint$confint.ploidy[1], cint$values.ploidy[select, 1], cint$confint.ploidy[2]), 
